@@ -24,8 +24,10 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <string.h>		/* For memset() */
 
+#include "gdkalias.h"
 #include "gdk.h"
 #include "gdkinternals.h"
 
@@ -558,6 +560,7 @@ gdk_event_get_time (GdkEvent *event)
       case GDK_UNMAP:
       case GDK_WINDOW_STATE:
       case GDK_SETTING:
+      case GDK_OWNER_CHANGE:
         /* return current time */
         break;
       }
@@ -633,6 +636,7 @@ gdk_event_get_state (GdkEvent        *event,
       case GDK_UNMAP:
       case GDK_WINDOW_STATE:
       case GDK_SETTING:
+      case GDK_OWNER_CHANGE:
         /* no state field */
         break;
       }
@@ -1038,7 +1042,7 @@ _gdk_event_button_generate (GdkDisplay *display,
       display->button_click_time[1] = 0;
       display->button_click_time[0] = 0;
       display->button_window[1] = NULL;
-      display->button_window[0] = 0;
+      display->button_window[0] = NULL;
       display->button_number[1] = -1;
       display->button_number[0] = -1;
       display->button_x[0] = display->button_x[1] = 0;
