@@ -31,8 +31,10 @@
 #include <config.h>
 
 #include <gdk/gdkkeysyms.h>
-#include "gtkaboutdialog.h"
 
+#include "gtkalias.h"
+
+#include "gtkaboutdialog.h"
 #include "gtkbutton.h"
 #include "gtkbbox.h"
 #include "gtkdialog.h"
@@ -100,7 +102,8 @@ enum
   PROP_DOCUMENTERS,
   PROP_TRANSLATOR_CREDITS,
   PROP_ARTISTS,
-  PROP_LOGO
+  PROP_LOGO,
+  PROP_LOGO_ICON_NAME
 };
 
 static void                 gtk_about_dialog_finalize       (GObject            *object);
@@ -153,7 +156,12 @@ static void                 display_license_dialog          (GtkWidget
 				 
 				 				       
 static GtkAboutDialogActivateLinkFunc activate_email_hook = NULL;
+static gpointer activate_email_hook_data = NULL;
+static GDestroyNotify activate_email_hook_destroy = NULL;
+
 static GtkAboutDialogActivateLinkFunc activate_url_hook = NULL;
+static gpointer activate_url_hook_data = NULL;
+static GDestroyNotify activate_url_hook_destroy = NULL;
 
 G_DEFINE_TYPE (GtkAboutDialog, gtk_about_dialog, GTK_TYPE_DIALOG);
 
@@ -173,6 +181,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 
   object_class->finalize = gtk_about_dialog_finalize;
 
+  /**
+   * GtkAboutDialog:name:
+   *
+   * The name of the program. 
+   * If this is not set, it defaults to g_get_application_name().
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_NAME,
 				   g_param_spec_string ("name",
@@ -181,6 +197,13 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:version:
+   *
+   * The version of the program. 
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_VERSION,
 				   g_param_spec_string ("version",
@@ -188,6 +211,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							P_("The version of the program"),
 							NULL,
 							G_PARAM_READWRITE));
+
+  /**
+   * GtkAboutDialog:copyright:
+   *
+   * Copyright information for the program. 
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_COPYRIGHT,
 				   g_param_spec_string ("copyright",
@@ -196,6 +227,16 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 	
+
+  /**
+   * GtkAboutDialog:comments:
+   *
+   * Comments about the program. This string is displayed in a label 
+   * in the main dialog, thus it should be a short explanation of
+   * the main purpose of the program, not a detailed list of features.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_COMMENTS,
 				   g_param_spec_string ("comments",
@@ -203,6 +244,16 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							P_("Comments about the program"),
 							NULL,
 							G_PARAM_READWRITE));
+
+  /**
+   * GtkAboutDialog:license:
+   *
+   * The license of the program. This string is displayed in a 
+   * text view in a secondary dialog, therefore it is fine to use
+   * a long multi-paragraph text.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_LICENSE,
 				   g_param_spec_string ("license",
@@ -211,6 +262,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:website:
+   *
+   * The URL for the link to the website of the program. 
+   * This should be a string starting with "http://.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_WEBSITE,
 				   g_param_spec_string ("website",
@@ -219,6 +278,15 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:website-label:
+   *
+   * The label for the link to the website of the program. If this is not set, 
+   * it defaults to the URL specified in the 
+   * <link linkend="GtkAboutDialog--website">website</link> property.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_WEBSITE_LABEL,
 				   g_param_spec_string ("website_label",
@@ -227,13 +295,32 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:authors:
+   *
+   * The authors of the program, as a %NULL-terminated array of strings.
+   * Each string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_AUTHORS,
 				   g_param_spec_boxed ("authors",
 						       P_("Authors"),
-						       P_("List of authors of the programs"),
+						       P_("List of authors of the program"),
 						       G_TYPE_STRV,
 						       G_PARAM_READWRITE));
+
+  /**
+   * GtkAboutDialog:documenters:
+   *
+   * The people documenting the program, as a %NULL-terminated array of strings.
+   * Each string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_DOCUMENTERS,
 				   g_param_spec_boxed ("documenters",
@@ -242,6 +329,15 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 						       G_TYPE_STRV,
 						       G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:artists:
+   *
+   * The people who contributed artwork to the program, as a %NULL-terminated array of strings.
+   * Each string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_ARTISTS,
 				   g_param_spec_boxed ("artists",
@@ -250,6 +346,16 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 						       G_TYPE_STRV,
 						       G_PARAM_READWRITE));
 
+
+  /**
+   * GtkAboutDialog:translator-credits:
+   *
+   * Credits to the translators. This string should be marked as translatable.
+   * The string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_TRANSLATOR_CREDITS,
 				   g_param_spec_string ("translator_credits",
@@ -258,6 +364,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 	
+  /**
+   * GtkAboutDialog:logo:
+   *
+   * A logo for the about box. If this is not set, it defaults to 
+   * gtk_window_get_default_icon_list().
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_LOGO,
 				   g_param_spec_object ("logo",
@@ -266,6 +380,22 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							GDK_TYPE_PIXBUF,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:logo-icon-name:
+   *
+   * A named icon to use as the logo for the about box. This property
+   * overrides the <link linkend="GtkAboutDialog--logo">logo</link> property.
+   *
+   * Since: 2.6
+   */  
+  g_object_class_install_property (object_class,
+				   PROP_LOGO_ICON_NAME,
+				   g_param_spec_string ("logo_icon_name",
+							P_("Logo Icon Name"),
+							P_("A named icon to use as the logo for the about box."),
+							NULL,
+							G_PARAM_READWRITE));
+
   /* Style properties */
   gtk_widget_class_install_style_property (widget_class,
                                            g_param_spec_boxed ("link_color",
@@ -389,8 +519,8 @@ gtk_about_dialog_finalize (GObject *object)
   g_strfreev (priv->documenters);
   g_strfreev (priv->artists);
 
-  g_free (priv);
-  about->private_data = NULL;
+  gdk_cursor_unref (priv->hand_cursor);
+  gdk_cursor_unref (priv->regular_cursor);
 
   G_OBJECT_CLASS (gtk_about_dialog_parent_class)->finalize (object);
 }
@@ -430,17 +560,20 @@ gtk_about_dialog_set_property (GObject      *object,
       gtk_about_dialog_set_logo (about, g_value_get_object (value));
       break; 
     case PROP_AUTHORS:
-      gtk_about_dialog_set_authors (about, (gchar**)g_value_get_boxed (value));
+      gtk_about_dialog_set_authors (about, (const gchar**)g_value_get_boxed (value));
       break;
     case PROP_DOCUMENTERS:
-      gtk_about_dialog_set_documenters (about, (gchar**)g_value_get_boxed (value));
+      gtk_about_dialog_set_documenters (about, (const gchar**)g_value_get_boxed (value));
       break;	
     case PROP_ARTISTS:
-      gtk_about_dialog_set_artists (about, (gchar**)g_value_get_boxed (value));
+      gtk_about_dialog_set_artists (about, (const gchar**)g_value_get_boxed (value));
       break;	
     case PROP_TRANSLATOR_CREDITS:
       gtk_about_dialog_set_translator_credits (about, g_value_get_string (value));
       break;
+    case PROP_LOGO_ICON_NAME:
+      gtk_about_dialog_set_logo_icon_name (about, g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -492,7 +625,21 @@ gtk_about_dialog_get_property (GObject    *object,
       g_value_set_boxed (value, priv->artists);
       break;
     case PROP_LOGO:
-      g_value_set_object (value, gtk_image_get_pixbuf (GTK_IMAGE (priv->logo_image)));
+      if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_PIXBUF)
+	g_value_set_object (value, gtk_image_get_pixbuf (GTK_IMAGE (priv->logo_image)));
+      else
+	g_value_set_object (value, NULL);
+      break;
+    case PROP_LOGO_ICON_NAME:
+      if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_ICON_NAME)
+	{
+	  const gchar *icon_name;
+
+	  gtk_image_get_icon_name (GTK_IMAGE (priv->logo_image), &icon_name, NULL);
+	  g_value_set_string (value, icon_name);
+	}
+      else
+	g_value_set_string (value, NULL);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -882,6 +1029,7 @@ gtk_about_dialog_set_website (GtkAboutDialog *about,
     {
       priv->website = NULL;
       g_object_set_data (G_OBJECT (priv->website_button), "url", NULL);
+      gtk_widget_hide (priv->website_button);
     }
   g_free (tmp);
 
@@ -967,7 +1115,7 @@ gtk_about_dialog_set_website_label (GtkAboutDialog *about,
  *
  * Since: 2.6
  **/
-gchar **
+G_CONST_RETURN gchar * G_CONST_RETURN *
 gtk_about_dialog_get_authors (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
@@ -976,7 +1124,7 @@ gtk_about_dialog_get_authors (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->authors;
+  return (const gchar * const *) priv->authors;
 }
 
 /**
@@ -991,7 +1139,7 @@ gtk_about_dialog_get_authors (GtkAboutDialog *about)
  **/
 void
 gtk_about_dialog_set_authors (GtkAboutDialog  *about, 
-			      gchar          **authors)
+			      const gchar    **authors)
 {
   GtkAboutDialogPrivate *priv;
   gchar **tmp;
@@ -1001,7 +1149,7 @@ gtk_about_dialog_set_authors (GtkAboutDialog  *about,
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
   tmp = priv->authors;
-  priv->authors = g_strdupv (authors);
+  priv->authors = g_strdupv ((gchar **)authors);
   g_strfreev (tmp);
 
   if (priv->authors != NULL)
@@ -1023,7 +1171,7 @@ gtk_about_dialog_set_authors (GtkAboutDialog  *about,
  *
  * Since: 2.6
  **/
-gchar **
+G_CONST_RETURN gchar * G_CONST_RETURN *
 gtk_about_dialog_get_documenters (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
@@ -1032,7 +1180,7 @@ gtk_about_dialog_get_documenters (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->documenters;
+  return (const gchar * const *)priv->documenters;
 }
 
 /**
@@ -1047,7 +1195,7 @@ gtk_about_dialog_get_documenters (GtkAboutDialog *about)
  **/
 void
 gtk_about_dialog_set_documenters (GtkAboutDialog *about, 
-				  gchar         **documenters)
+				  const gchar   **documenters)
 {
   GtkAboutDialogPrivate *priv;
   gchar **tmp;
@@ -1057,7 +1205,7 @@ gtk_about_dialog_set_documenters (GtkAboutDialog *about,
   priv = (GtkAboutDialogPrivate *)about->private_data;
   
   tmp = priv->documenters;
-  priv->documenters = g_strdupv (documenters);
+  priv->documenters = g_strdupv ((gchar **)documenters);
   g_strfreev (tmp);
 
   if (priv->documenters != NULL)
@@ -1079,7 +1227,7 @@ gtk_about_dialog_set_documenters (GtkAboutDialog *about,
  *
  * Since: 2.6
  **/
-gchar **
+G_CONST_RETURN gchar * G_CONST_RETURN *
 gtk_about_dialog_get_artists (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
@@ -1088,7 +1236,7 @@ gtk_about_dialog_get_artists (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->artists;
+  return (const gchar * const *)priv->artists;
 }
 
 /**
@@ -1103,7 +1251,7 @@ gtk_about_dialog_get_artists (GtkAboutDialog *about)
  **/
 void
 gtk_about_dialog_set_artists (GtkAboutDialog *about, 
-			      gchar         **artists)
+			      const gchar   **artists)
 {
   GtkAboutDialogPrivate *priv;
   gchar **tmp;
@@ -1113,7 +1261,7 @@ gtk_about_dialog_set_artists (GtkAboutDialog *about,
   priv = (GtkAboutDialogPrivate *)about->private_data;
   
   tmp = priv->artists;
-  priv->artists = g_strdupv (artists);
+  priv->artists = g_strdupv ((gchar **)artists);
   g_strfreev (tmp);
 
   if (priv->artists != NULL)
@@ -1159,11 +1307,11 @@ gtk_about_dialog_get_translator_credits (GtkAboutDialog *about)
  * Using gettext(), a simple way to achieve that is to mark the
  * string for translation:
  * <informalexample><programlisting>
- *  gtk_about_dialog_set_translator_credits (about, _("translator_credits"));
+ *  gtk_about_dialog_set_translator_credits (about, _("translator-credits"));
  * </programlisting></informalexample>
- * It is a good idea to use the customary msgid "translator_credits" for this
+ * It is a good idea to use the customary msgid "translator-credits" for this
  * purpose, since translators will already know the purpose of that msgid, and
- * since #GtkAboutDialog will detect if "translator_credits" is untranslated
+ * since #GtkAboutDialog will detect if "translator-credits" is untranslated
  * and hide the tab.
  *
  * Since: 2.6
@@ -1210,7 +1358,10 @@ gtk_about_dialog_get_logo (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return gtk_image_get_pixbuf (GTK_IMAGE (priv->logo_image));
+  if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_PIXBUF)
+    return gtk_image_get_pixbuf (GTK_IMAGE (priv->logo_image));
+  else
+    return NULL;
 }
 
 static GtkIconSet *
@@ -1225,6 +1376,7 @@ icon_set_new_from_pixbufs (GList *pixbufs)
       GtkIconSource *icon_source = gtk_icon_source_new ();
       gtk_icon_source_set_pixbuf (icon_source, pixbuf);
       gtk_icon_set_add_source (icon_set, icon_source);
+      gtk_icon_source_free (icon_source);
     }
   
   return icon_set;
@@ -1252,6 +1404,11 @@ gtk_about_dialog_set_logo (GtkAboutDialog *about,
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
+  g_object_freeze_notify (G_OBJECT (about));
+
+  if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_ICON_NAME)
+    g_object_notify (G_OBJECT (about), "logo_icon_name");
+
   if (logo != NULL) 
     gtk_image_set_from_pixbuf (GTK_IMAGE (priv->logo_image), logo);
   else 
@@ -1271,6 +1428,70 @@ gtk_about_dialog_set_logo (GtkAboutDialog *about,
     }
 
   g_object_notify (G_OBJECT (about), "logo");
+
+  g_object_thaw_notify (G_OBJECT (about));
+}
+
+/**
+ * gtk_about_dialog_get_logo_icon_name:
+ * @about: a #GtkAboutDialog
+ * 
+ * Returns the icon name displayed as logo in the about dialog.
+ * 
+ * Return value: the icon name displayed as logo. The string is
+ *   owned by the about dialog. If you want to keep a reference
+ *   to it, you have to call g_strdup() on it.
+ *
+ * Since: 2.6
+ **/
+G_CONST_RETURN gchar *
+gtk_about_dialog_get_logo_icon_name (GtkAboutDialog *about)
+{
+  GtkAboutDialogPrivate *priv;
+  const gchar *icon_name;
+  
+  g_return_val_if_fail (GTK_IS_ABOUT_DIALOG (about), NULL);
+
+  priv = (GtkAboutDialogPrivate *)about->private_data;
+
+  if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_ICON_NAME)
+    gtk_image_get_icon_name (GTK_IMAGE (priv->logo_image), &icon_name, NULL);
+
+  return icon_name;
+}
+
+/**
+ * gtk_about_dialog_set_logo_icon_name:
+ * @about: a #GtkAboutDialog
+ * @icon_name: an icon name, or %NULL
+ * 
+ * Sets the pixbuf to be displayed as logo in 
+ * the about dialog. If it is %NULL, the default
+ * window icon set with gtk_window_set_default_icon()
+ * will be used.
+ *
+ * Since: 2.6
+ **/
+void
+gtk_about_dialog_set_logo_icon_name (GtkAboutDialog *about,
+				     const gchar    *icon_name)
+{
+  GtkAboutDialogPrivate *priv;
+
+  g_return_if_fail (GTK_IS_ABOUT_DIALOG (about));
+
+  priv = (GtkAboutDialogPrivate *)about->private_data;
+
+  g_object_freeze_notify (G_OBJECT (about));
+
+  if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_PIXBUF)
+    g_object_notify (G_OBJECT (about), "logo");
+
+  gtk_image_set_from_icon_name (GTK_IMAGE (priv->logo_image), icon_name,
+				GTK_ICON_SIZE_DIALOG);
+  g_object_notify (G_OBJECT (about), "logo_icon_name");
+
+  g_object_thaw_notify (G_OBJECT (about));
 }
 
 static void
@@ -1281,7 +1502,7 @@ activate_url (GtkWidget *widget,
   gchar *url = g_object_get_data (G_OBJECT (widget), "url");
   
   if (activate_url_hook != NULL)
-    (* activate_url_hook) (about, url);
+    (* activate_url_hook) (about, url, activate_url_hook_data);
 }
 
 static void
@@ -1346,13 +1567,13 @@ follow_if_link (GtkAboutDialog *about,
 
       if (email != NULL && activate_email_hook != NULL)
         {
-	  (* activate_email_hook) (about, email);
+	  (* activate_email_hook) (about, email, activate_email_hook_data);
 	  break;
         }
 
       if (url != NULL && activate_url_hook != NULL)
         {
-	  (* activate_url_hook) (about, url);
+	  (* activate_url_hook) (about, url, activate_url_hook_data);
 	  break;
         }
     }
@@ -1520,7 +1741,6 @@ add_credits_page (GtkAboutDialog *about,
   gchar *q0, *q1, *q2, *r1, *r2;
   GtkWidget *sw, *view;
   GtkTextBuffer *buffer;
-  GtkStyle *style;
   gboolean linkify_email, linkify_urls;
   GdkColor *style_link_color;
   GdkColor link_color = { 0, 0, 0, 0xffff };
@@ -1682,9 +1902,10 @@ display_credits_dialog (GtkWidget *button,
   if (priv->documenters != NULL)
     add_credits_page (about, notebook, _("Documented by"), priv->documenters);
     
+  /* Don't show an untranslated gettext msgid */
   if (priv->translator_credits != NULL &&
-      !strcmp (priv->translator_credits, "translator_credits") &&
-      !strcmp (priv->translator_credits, "translator-credits")) 
+      strcmp (priv->translator_credits, "translator_credits") &&
+      strcmp (priv->translator_credits, "translator-credits")) 
     {
       gchar *translators[2];
       
@@ -1756,6 +1977,27 @@ display_license_dialog (GtkWidget *button,
   gtk_widget_show_all (dialog);
 }
 
+static void 
+close_cb (GtkAboutDialog *about)
+{
+  GtkAboutDialogPrivate *priv = (GtkAboutDialogPrivate *)about->private_data;
+
+  if (priv->license_dialog != NULL)
+    {
+      gtk_widget_destroy (priv->license_dialog);
+      priv->license_dialog = NULL;
+    }
+
+  if (priv->credits_dialog != NULL)
+    {
+      gtk_widget_destroy (priv->credits_dialog);
+      priv->credits_dialog = NULL;
+    }
+
+  gtk_widget_hide (GTK_WIDGET (about));
+  
+}
+
 /**
  * gtk_about_dialog_new:
  *
@@ -1777,7 +2019,7 @@ gtk_about_dialog_new (void)
   /* Close dialog on user response */
   g_signal_connect (G_OBJECT (dialog),
                     "response",
-                    G_CALLBACK (gtk_widget_hide),
+                    G_CALLBACK (close_cb),
                     NULL);
 
   return GTK_WIDGET (dialog);
@@ -1786,6 +2028,8 @@ gtk_about_dialog_new (void)
 /**
  * gtk_about_dialog_set_email_hook:
  * @func: a function to call when an email link is activated.
+ * @data: data to pass to @func
+ * @destroy: #GDestroyNotify for @data
  * 
  * Installs a global function to be called whenever the user activates an
  * email link in an about dialog. 
@@ -1795,13 +2039,20 @@ gtk_about_dialog_new (void)
  * Since: 2.6
  */
 GtkAboutDialogActivateLinkFunc      
-gtk_about_dialog_set_email_hook (GtkAboutDialogActivateLinkFunc func)
+gtk_about_dialog_set_email_hook (GtkAboutDialogActivateLinkFunc func, 
+				 gpointer                       data, 
+				 GDestroyNotify                 destroy)
 {
   GtkAboutDialogActivateLinkFunc old;
 
+  if (activate_email_hook_destroy != NULL)
+    (* activate_email_hook_destroy) (activate_url_hook_data);
+
   old = activate_email_hook;
 
   activate_email_hook = func;
+  activate_email_hook_data = data;
+  activate_email_hook_destroy = destroy;
 
   return old;
 }
@@ -1809,6 +2060,8 @@ gtk_about_dialog_set_email_hook (GtkAboutDialogActivateLinkFunc func)
 /**
  * gtk_about_dialog_set_url_hook:
  * @func: a function to call when a URL link is activated.
+ * @data: data to pass to @func
+ * @destroy: #GDestroyNotify for @data
  * 
  * Installs a global function to be called whenever the user activates a
  * URL link in an about dialog.
@@ -1818,13 +2071,20 @@ gtk_about_dialog_set_email_hook (GtkAboutDialogActivateLinkFunc func)
  * Since: 2.6
  */
 GtkAboutDialogActivateLinkFunc      
-gtk_about_dialog_set_url_hook (GtkAboutDialogActivateLinkFunc func)
+gtk_about_dialog_set_url_hook (GtkAboutDialogActivateLinkFunc func, 
+			       gpointer                       data, 
+			       GDestroyNotify                 destroy)
 {
   GtkAboutDialogActivateLinkFunc old;
 
+  if (activate_url_hook_destroy != NULL)
+    (* activate_url_hook_destroy) (activate_email_hook_data);
+
   old = activate_url_hook;
 
   activate_url_hook = func;
+  activate_url_hook_data = data;
+  activate_url_hook_destroy = destroy;
 
   return old;
 }
@@ -1836,6 +2096,8 @@ gtk_about_dialog_set_url_hook (GtkAboutDialogActivateLinkFunc func)
  * @Varargs: value of first property, followed by more properties, %NULL-terminated
  *
  * This is a convenience function for showing an application's about box.
+ * The constructed dialog is associated with the parent window and 
+ * reused for future invocations of this function.
  *
  * Since: 2.6
  */
@@ -1869,6 +2131,7 @@ gtk_show_about_dialog (GtkWindow   *parent,
       if (parent) 
 	{
 	  gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+	  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 	  g_object_set_data_full (G_OBJECT (parent), "gtk-about-dialog", 
 				  dialog, g_object_unref);
 	}
@@ -1879,10 +2142,3 @@ gtk_show_about_dialog (GtkWindow   *parent,
   
   gtk_window_present (GTK_WINDOW (dialog));
 }
-
-
-
-
-
-
-
