@@ -24,7 +24,9 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <string.h>
+#include "gtkalias.h"
 #include "gtkalignment.h"
 #include "gtkbutton.h"
 #include "gtklabel.h"
@@ -77,6 +79,7 @@ struct _GtkButtonPrivate
   GtkSettings *settings;
   guint        show_image_connection;
   GtkWidget   *image;
+  guint        align_set : 1;
 };
 
 static void gtk_button_class_init     (GtkButtonClass   *klass);
@@ -122,10 +125,15 @@ static GType gtk_button_child_type    (GtkContainer     *container);
 static void gtk_button_finish_activate (GtkButton *button,
 					gboolean   do_it);
 
-static GObject*	gtk_button_constructor     (GType                  type,
-					    guint                  n_construct_properties,
-					    GObjectConstructParam *construct_params);
+static GObject*	gtk_button_constructor (GType                  type,
+					guint                  n_construct_properties,
+					GObjectConstructParam *construct_params);
 static void gtk_button_construct_child (GtkButton             *button);
+static void gtk_button_state_changed   (GtkWidget             *widget,
+					GtkStateType           previous_state);
+static void gtk_button_grab_notify     (GtkWidget             *widget,
+					gboolean               was_grabbed);
+
 
 
 static GtkBinClass *parent_class = NULL;
@@ -193,8 +201,11 @@ gtk_button_class_init (GtkButtonClass *klass)
   widget_class->key_release_event = gtk_button_key_release;
   widget_class->enter_notify_event = gtk_button_enter_notify;
   widget_class->leave_notify_event = gtk_button_leave_notify;
+  widget_class->state_changed = gtk_button_state_changed;
+  widget_class->grab_notify = gtk_button_grab_notify;
 
   container_class->child_type = gtk_button_child_type;
+  container_class->add = gtk_button_add;
 
   klass->pressed = gtk_real_button_pressed;
   klass->released = gtk_real_button_released;
@@ -325,6 +336,7 @@ gtk_button_class_init (GtkButtonClass *klass)
 
   /**
    * GtkButton::activate:
+   * @widget: the object which received the signal.
    *
    * The "activate" signal on GtkButton is an action signal and
    * emitting it causes the button to animate press then release. 
@@ -371,6 +383,21 @@ gtk_button_class_init (GtkButtonClass *klass)
 							     0,
 							     G_PARAM_READABLE));
 
+  /**
+   * GtkButton:displace-focus:
+   *
+   * Whether the child_displacement_x/child_displacement_y properties should also 
+   * affect the focus rectangle.
+   *
+   * Since: 2.6
+   */
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_boolean ("displace-focus",
+								 P_("Displace focus"),
+								 P_("Whether the child_displacement_x/_y properties should also affect the focus rectangle"),
+						       FALSE,
+						       G_PARAM_READABLE));
+
   gtk_settings_install_property (g_param_spec_boolean ("gtk-button-images",
 						       P_("Show button images"),
 						       P_("Whether stock icons should be shown in buttons"),
@@ -402,6 +429,7 @@ gtk_button_init (GtkButton *button)
 
   priv->xalign = 0.5;
   priv->yalign = 0.5;
+  priv->align_set = 0;
 }
 
 static void
@@ -450,19 +478,25 @@ gtk_button_child_type  (GtkContainer     *container)
 }
 
 static void
-maybe_set_alignment (GtkWidget *widget,
-		     gfloat     xalign,
-		     gfloat     yalign)
+maybe_set_alignment (GtkButton *button,
+		     GtkWidget *widget)
 {
+  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
+
   if (GTK_IS_MISC (widget))
     {
       GtkMisc *misc = GTK_MISC (widget);
-      gtk_misc_set_alignment (misc, xalign, yalign);
+      
+      if (priv->align_set)
+	gtk_misc_set_alignment (misc, priv->xalign, priv->yalign);
     }
   else if (GTK_IS_ALIGNMENT (widget))
     {
       GtkAlignment *alignment = GTK_ALIGNMENT (widget);
-      gtk_alignment_set (alignment, xalign, yalign, alignment->xscale, alignment->yscale);
+
+      if (priv->align_set)
+	gtk_alignment_set (alignment, priv->xalign, priv->yalign, 
+			   alignment->xscale, alignment->yscale);
     }
 }
 
@@ -470,10 +504,8 @@ static void
 gtk_button_add (GtkContainer *container,
 		GtkWidget    *widget)
 {
-  GtkButton *button = GTK_BUTTON (container);
-  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
+  maybe_set_alignment (GTK_BUTTON (container), widget);
 
-  maybe_set_alignment (widget, priv->xalign, priv->yalign);
   GTK_CONTAINER_CLASS (parent_class)->add (container, widget);
 }
 
@@ -586,8 +618,11 @@ gtk_button_construct_child (GtkButton *button)
     return;
 
   if (GTK_BIN (button)->child)
-    gtk_container_remove (GTK_CONTAINER (button),
-			  GTK_BIN (button)->child);
+    {
+      gtk_container_remove (GTK_CONTAINER (button),
+  			    GTK_BIN (button)->child);
+      priv->image = NULL;
+    }
   
   if (button->use_stock &&
       gtk_stock_lookup (button->label_text, &item))
@@ -603,8 +638,11 @@ gtk_button_construct_child (GtkButton *button)
 		    NULL);
       hbox = gtk_hbox_new (FALSE, 2);
 
-      align = gtk_alignment_new (priv->xalign, priv->yalign, 0.0, 0.0);
-      
+      if (priv->align_set)
+	align = gtk_alignment_new (priv->xalign, priv->yalign, 0.0, 0.0);
+      else
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	
       gtk_box_pack_start (GTK_BOX (hbox), priv->image, FALSE, FALSE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
       
@@ -623,7 +661,8 @@ gtk_button_construct_child (GtkButton *button)
   else
     label = gtk_label_new (button->label_text);
   
-  gtk_misc_set_alignment (GTK_MISC (label), priv->xalign, priv->yalign);
+  if (priv->align_set)
+    gtk_misc_set_alignment (GTK_MISC (label), priv->xalign, priv->yalign);
 
   gtk_widget_show (label);
   gtk_container_add (GTK_CONTAINER (button), label);
@@ -802,8 +841,6 @@ gtk_button_map (GtkWidget *widget)
 {
   GtkButton *button = GTK_BUTTON (widget);
   
-  g_return_if_fail (GTK_IS_BUTTON (widget));
-
   GTK_WIDGET_CLASS (parent_class)->map (widget);
 
   if (button->event_window)
@@ -815,8 +852,6 @@ gtk_button_unmap (GtkWidget *widget)
 {
   GtkButton *button = GTK_BUTTON (widget);
     
-  g_return_if_fail (GTK_IS_BUTTON (widget));
-
   if (button->event_window)
     gdk_window_hide (button->event_window);
 
@@ -868,11 +903,10 @@ gtk_button_size_request (GtkWidget      *widget,
 {
   GtkButton *button = GTK_BUTTON (widget);
   GtkBorder default_border;
-  gboolean interior_focus;
   gint focus_width;
   gint focus_pad;
 
-  gtk_button_get_props (button, &default_border, NULL, &interior_focus);
+  gtk_button_get_props (button, &default_border, NULL, NULL);
   gtk_widget_style_get (GTK_WIDGET (widget),
 			"focus-line-width", &focus_width,
 			"focus-padding", &focus_pad,
@@ -914,8 +948,15 @@ gtk_button_size_allocate (GtkWidget     *widget,
   gint xthickness = GTK_WIDGET (widget)->style->xthickness;
   gint ythickness = GTK_WIDGET (widget)->style->ythickness;
   GtkBorder default_border;
+  gint focus_width;
+  gint focus_pad;
 
   gtk_button_get_props (button, &default_border, NULL, NULL);
+  gtk_widget_style_get (GTK_WIDGET (widget),
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_pad,
+			NULL);
+ 
 			    
   widget->allocation = *allocation;
 
@@ -944,6 +985,14 @@ gtk_button_size_allocate (GtkWidget     *widget,
 	  child_allocation.height = MAX (1, child_allocation.height - default_border.top - default_border.bottom);
 	}
 
+      if (GTK_WIDGET_CAN_FOCUS (button))
+	{
+	  child_allocation.x += focus_width + focus_pad;
+	  child_allocation.y += focus_width + focus_pad;
+	  child_allocation.width =  MAX (1, child_allocation.width - (focus_width + focus_pad) * 2);
+	  child_allocation.height = MAX (1, child_allocation.height - (focus_width + focus_pad) * 2);
+	}
+
       if (button->depressed)
 	{
 	  gint child_displacement_x;
@@ -1033,12 +1082,22 @@ _gtk_button_paint (GtkButton    *button,
        
       if (GTK_WIDGET_HAS_FOCUS (widget))
 	{
+	  gint child_displacement_x;
+	  gint child_displacement_y;
+	  gboolean displace_focus;
+	  
+	  gtk_widget_style_get (GTK_WIDGET (widget),
+				"child_displacement_y", &child_displacement_y,
+				"child_displacement_x", &child_displacement_x,
+				"displace_focus", &displace_focus,
+				NULL);
+
 	  if (interior_focus)
 	    {
 	      x += widget->style->xthickness + focus_pad;
 	      y += widget->style->ythickness + focus_pad;
 	      width -= 2 * (widget->style->xthickness + focus_pad);
-	      height -=  2 * (widget->style->xthickness + focus_pad);
+	      height -=  2 * (widget->style->ythickness + focus_pad);
 	    }
 	  else
 	    {
@@ -1048,6 +1107,12 @@ _gtk_button_paint (GtkButton    *button,
 	      height += 2 * (focus_width + focus_pad);
 	    }
 
+	  if (button->depressed && displace_focus)
+	    {
+	      x += child_displacement_x;
+	      y += child_displacement_y;
+	    }
+
 	  gtk_paint_focus (widget->style, widget->window, GTK_WIDGET_STATE (widget),
 			   area, widget, "button",
 			   x, y, width, height);
@@ -1389,7 +1454,7 @@ gtk_button_get_use_stock (GtkButton *button)
 /**
  * gtk_button_set_focus_on_click:
  * @button: a #GtkButton
- * @focus_on_click: whether the mouse grabs focus when clicked with the mouse
+ * @focus_on_click: whether the button grabs focus when clicked with the mouse
  * 
  * Sets whether the button will grab focus when it is clicked with the mouse.
  * Making mouse clicks not grab focus is useful in places like toolbars where
@@ -1460,8 +1525,9 @@ gtk_button_set_alignment (GtkButton *button,
 
   priv->xalign = xalign;
   priv->yalign = yalign;
+  priv->align_set = 1;
 
-  maybe_set_alignment (GTK_BIN (button)->child, xalign, yalign);
+  maybe_set_alignment (button, GTK_BIN (button)->child);
 
   g_object_freeze_notify (G_OBJECT (button));
   g_object_notify (G_OBJECT (button), "xalign");
@@ -1587,3 +1653,29 @@ gtk_button_screen_changed (GtkWidget *widget,
 
   show_image_change_notify (GTK_BUTTON (widget));
 }
+
+static void
+gtk_button_state_changed (GtkWidget    *widget,
+                          GtkStateType  previous_state)
+{
+  GtkButton *button = GTK_BUTTON (widget);
+
+  if (!GTK_WIDGET_IS_SENSITIVE (widget))
+    {
+      button->in_button = FALSE;
+      gtk_real_button_released (button);
+    }
+}
+
+static void
+gtk_button_grab_notify (GtkWidget *widget,
+			gboolean   was_grabbed)
+{
+  GtkButton *button = GTK_BUTTON (widget);
+
+  if (!was_grabbed)
+    {
+      button->in_button = FALSE;
+      gtk_real_button_released (button);
+    }
+}
