@@ -18,6 +18,7 @@
  */
 
 #include <config.h>
+#include "gtkalias.h"
 #include "gtkcellview.h"
 #include "gtkcelllayout.h"
 #include "gtkintl.h"
@@ -80,7 +81,6 @@ static void        gtk_cell_view_set_valuesv              (GtkCellView      *cel
                                                            va_list           args);
 static GtkCellViewCellInfo *gtk_cell_view_get_cell_info   (GtkCellView      *cellview,
                                                            GtkCellRenderer  *renderer);
-static void        gtk_cell_view_set_cell_data            (GtkCellView      *cellview);
 
 
 static void        gtk_cell_view_cell_layout_pack_start        (GtkCellLayout         *layout,
@@ -304,8 +304,7 @@ gtk_cell_view_finalize (GObject *object)
   if (cellview->priv->displayed_row)
      gtk_tree_row_reference_free (cellview->priv->displayed_row);
 
-  if (G_OBJECT_CLASS (parent_class)->finalize)
-    (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+  (* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 static void
@@ -391,7 +390,7 @@ gtk_cell_view_size_allocate (GtkWidget     *widget,
       if (!info->cell->visible)
         continue;
 
-      info->real_width = info->requested_width + (info->expand ? extra_space : 0);
+      info->real_width = info->requested_width + (info->expand?extra_space:0);
     }
 
   /* iterate list for PACK_END cells */
@@ -405,7 +404,7 @@ gtk_cell_view_size_allocate (GtkWidget     *widget,
       if (!info->cell->visible)
         continue;
 
-      info->real_width = info->requested_width + (info->expand ? extra_space : 0);
+      info->real_width = info->requested_width + (info->expand?extra_space:0);
     }
 }
 
@@ -535,20 +534,23 @@ gtk_cell_view_get_cell_info (GtkCellView     *cellview,
   return NULL;
 }
 
-static void
-gtk_cell_view_set_cell_data (GtkCellView *cellview)
+void
+gtk_cell_view_set_cell_data (GtkCellView *cell_view)
 {
   GList *i;
   GtkTreeIter iter;
   GtkTreePath *path;
 
-  g_return_if_fail (cellview->priv->displayed_row != NULL);
-  
-  path = gtk_tree_row_reference_get_path (cellview->priv->displayed_row);
-  gtk_tree_model_get_iter (cellview->priv->model, &iter, path);
+  g_return_if_fail (cell_view->priv->displayed_row != NULL);
+
+  path = gtk_tree_row_reference_get_path (cell_view->priv->displayed_row);
+  if (!path)
+    return;
+
+  gtk_tree_model_get_iter (cell_view->priv->model, &iter, path);
   gtk_tree_path_free (path);
 
-  for (i = cellview->priv->cell_list; i; i = i->next)
+  for (i = cell_view->priv->cell_list; i; i = i->next)
     {
       GSList *j;
       GtkCellViewCellInfo *info = i->data;
@@ -561,7 +563,7 @@ gtk_cell_view_set_cell_data (GtkCellView *cellview)
           gint column = GPOINTER_TO_INT (j->next->data);
           GValue value = {0, };
 
-          gtk_tree_model_get_value (cellview->priv->model, &iter,
+          gtk_tree_model_get_value (cell_view->priv->model, &iter,
                                     column, &value);
           g_object_set_property (G_OBJECT (info->cell),
                                  property, &value);
@@ -569,9 +571,9 @@ gtk_cell_view_set_cell_data (GtkCellView *cellview)
         }
 
       if (info->func)
-	(* info->func) (GTK_CELL_LAYOUT (cellview),
+	(* info->func) (GTK_CELL_LAYOUT (cell_view),
 			info->cell,
-			cellview->priv->model,
+			cell_view->priv->model,
 			&iter,
 			info->func_data);
 
@@ -747,7 +749,15 @@ gtk_cell_view_cell_layout_reorder (GtkCellLayout   *layout,
   gtk_widget_queue_draw (GTK_WIDGET (cellview));
 }
 
-/* public API */
+/**
+ * gtk_cell_view_new:
+ *
+ * Creates a new #GtkCellView widget.
+ *
+ * Return value: A newly created #GtkCellView widget.
+ *
+ * Since: 2.6
+ */
 GtkWidget *
 gtk_cell_view_new (void)
 {
@@ -758,6 +768,17 @@ gtk_cell_view_new (void)
   return GTK_WIDGET (cellview);
 }
 
+/**
+ * gtk_cell_view_new_with_text:
+ * @text: the text to display in the cell view
+ *
+ * Creates a new #GtkCellView widget, adds a #GtkCellRendererText 
+ * to it, and makes its show @text.
+ *
+ * Return value: A newly created #GtkCellView widget.
+ *
+ * Since: 2.6
+ */
 GtkWidget *
 gtk_cell_view_new_with_text (const gchar *text)
 {
@@ -779,6 +800,19 @@ gtk_cell_view_new_with_text (const gchar *text)
   return GTK_WIDGET (cellview);
 }
 
+/**
+ * gtk_cell_view_new_with_markup:
+ * @markup: the text to display in the cell view
+ *
+ * Creates a new #GtkCellView widget, adds a #GtkCellRendererText 
+ * to it, and makes its show @markup. The text can text can be
+ * marked up with the <link linkend="PangoMarkupFormat">Pango text 
+ * markup language</link>.
+ *
+ * Return value: A newly created #GtkCellView widget.
+ *
+ * Since: 2.6
+ */
 GtkWidget *
 gtk_cell_view_new_with_markup (const gchar *markup)
 {
@@ -800,6 +834,17 @@ gtk_cell_view_new_with_markup (const gchar *markup)
   return GTK_WIDGET (cellview);
 }
 
+/**
+ * gtk_cell_view_new_with_pixbuf:
+ * @pixbuf: the image to display in the cell view
+ *
+ * Creates a new #GtkCellView widget, adds a #GtkCellRendererPixbuf 
+ * to it, and makes its show @pixbuf. 
+ *
+ * Return value: A newly created #GtkCellView widget.
+ *
+ * Since: 2.6
+ */
 GtkWidget *
 gtk_cell_view_new_with_pixbuf (GdkPixbuf *pixbuf)
 {
@@ -821,6 +866,18 @@ gtk_cell_view_new_with_pixbuf (GdkPixbuf *pixbuf)
   return GTK_WIDGET (cellview);
 }
 
+/**
+ * gtk_cell_view_set_value:
+ * @cell_view: a #GtkCellView widget
+ * @renderer: one of the renderers of @cell_view
+ * @property: the name of the property of @renderer to set
+ * @value: the new value to set the property to
+ * 
+ * Sets a property of a cell renderer of @cell_view, and
+ * makes sure the display of @cell_view is updated.
+ *
+ * Since: 2.6
+ */
 void
 gtk_cell_view_set_value (GtkCellView     *cell_view,
                          GtkCellRenderer *renderer,
@@ -855,6 +912,18 @@ gtk_cell_view_set_valuesv (GtkCellView     *cell_view,
     }
 }
 
+/**
+ * gtk_cell_view_set_values:
+ * @cell_view: a #GtkCellView widget
+ * @renderer: one of the renderers of @cell_view
+ * @Varargs: a list of pairs of property names and #GValue<!-- -->s,
+ *   finished by %NULL
+ * 
+ * Sets multiple properties of a cell renderer of @cell_view, and
+ * makes sure the display of @cell_view is updated.
+ *
+ * Since: 2.6
+ */
 void
 gtk_cell_view_set_values (GtkCellView     *cell_view,
                           GtkCellRenderer *renderer,
@@ -871,6 +940,17 @@ gtk_cell_view_set_values (GtkCellView     *cell_view,
   va_end (args);
 }
 
+/**
+ * gtk_cell_view_set_model:
+ * @cell_view: a #GtkCellView
+ * @model: a #GtkTreeModel
+ *
+ * Sets the model for @cell_view.  If @cell_view already has a model
+ * set, it will remove it before setting the new model.  If @model is 
+ * %NULL, then it will unset the old model.
+ *
+ * Since: 2.6
+ */
 void
 gtk_cell_view_set_model (GtkCellView  *cell_view,
                          GtkTreeModel *model)
@@ -905,6 +985,8 @@ gtk_cell_view_set_model (GtkCellView  *cell_view,
  * this is not normally a desired result, but may be
  * a needed intermediate state if say, the model for
  * the #GtkCellView becomes temporarily empty.
+ *
+ * Since: 2.6
  **/
 void
 gtk_cell_view_set_displayed_row (GtkCellView *cell_view,
@@ -940,6 +1022,19 @@ gtk_cell_view_get_displayed_row (GtkCellView *cell_view)
   return gtk_tree_row_reference_get_path (cell_view->priv->displayed_row);
 }
 
+/**
+ * gtk_cell_view_get_size_of_row:
+ * @cell_view: a #GtkCellView
+ * @path: a #GtkTreePath 
+ * @requisition: return location for the size 
+ *
+ * Sets @requisition to the size needed by @cell_view to display 
+ * the model row pointed to by @path.
+ * 
+ * Return value: %TRUE
+ *
+ * Since: 2.6
+ */
 gboolean
 gtk_cell_view_get_size_of_row (GtkCellView    *cell_view,
                                GtkTreePath    *path,
@@ -967,28 +1062,66 @@ gtk_cell_view_get_size_of_row (GtkCellView    *cell_view,
   return TRUE;
 }
 
+/**
+ * gtk_cell_view_set_background_color:
+ * @cell_view: a #GtkCellView
+ * @color: the new background color
+ *
+ * Sets the background color of @view.
+ *
+ * Since: 2.6
+ */
 void
-gtk_cell_view_set_background_color (GtkCellView    *view,
+gtk_cell_view_set_background_color (GtkCellView    *cell_view,
                                     const GdkColor *color)
 {
-  g_return_if_fail (GTK_IS_CELL_VIEW (view));
+  g_return_if_fail (GTK_IS_CELL_VIEW (cell_view));
 
   if (color)
     {
-      if (!view->priv->background_set)
+      if (!cell_view->priv->background_set)
         {
-          view->priv->background_set = TRUE;
-          g_object_notify (G_OBJECT (view), "background_set");
+          cell_view->priv->background_set = TRUE;
+          g_object_notify (G_OBJECT (cell_view), "background_set");
         }
 
-      view->priv->background = *color;
+      cell_view->priv->background = *color;
     }
   else
     {
-      if (view->priv->background_set)
+      if (cell_view->priv->background_set)
         {
-          view->priv->background_set = FALSE;
-          g_object_notify (G_OBJECT (view), "background_set");
+          cell_view->priv->background_set = FALSE;
+          g_object_notify (G_OBJECT (cell_view), "background_set");
         }
     }
 }
+
+/**
+ * gtk_cell_view_get_cell_renderers:
+ * @cell_view: a #GtkCellView
+ * 
+ * Returns the cell renderers which have been added to @cell_view.
+ *
+ * Return value: a list of cell renderers. The list, but not the
+ *   renderers has been newly allocated and should be freed with
+ *   g_list_free() when no longer needed.
+ * 
+ * Since: 2.6
+ */
+GList *
+gtk_cell_view_get_cell_renderers (GtkCellView *cell_view)
+{
+  GList *retval = NULL, *list;
+
+  g_return_val_if_fail (cell_view != NULL, NULL);
+
+  for (list = cell_view->priv->cell_list; list; list = list->next)
+    {
+      GtkCellViewCellInfo *info = (GtkCellViewCellInfo *)list->data;
+
+      retval = g_list_prepend (retval, info->cell);
+    }
+
+  return g_list_reverse (retval);
+}
