@@ -33,7 +33,7 @@ G_BEGIN_DECLS
 #define GTK_ICON_THEME_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_ICON_THEME, GtkIconThemeClass))
 #define GTK_IS_ICON_THEME(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_ICON_THEME))
 #define GTK_IS_ICON_THEME_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_ICON_THEME))
-#define GTK_ICON_THEME_GET_CLASS(obj)   (G_TYPE_CHECK_GET_CLASS ((obj), GTK_TYPE_ICON_THEME, GtkIconThemeClass))
+#define GTK_ICON_THEME_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_ICON_THEME, GtkIconThemeClass))
 
 typedef struct _GtkIconInfo         GtkIconInfo;
 typedef struct _GtkIconTheme        GtkIconTheme;
@@ -64,8 +64,8 @@ struct _GtkIconThemeClass
  *   Cannot be used together with %GTK_ICON_LOOKUP_NO_SVG.
  * @GTK_ICON_LOOKUP_USE_BUILTIN: When passed to
  *   gtk_icon_theme_lookup_icon() includes builtin icons
- *   as well as files. For a builtin icon, gdk_icon_info_get_filename()
- *   returns %NULL and you need to call gdk_icon_info_get_builtin_pixbuf().
+ *   as well as files. For a builtin icon, gtk_icon_info_get_filename()
+ *   returns %NULL and you need to call gtk_icon_info_get_builtin_pixbuf().
  * 
  * Used to specify options for gtk_icon_theme_lookup_icon()
  **/
@@ -116,6 +116,8 @@ void          gtk_icon_theme_set_custom_theme      (GtkIconTheme
 
 gboolean      gtk_icon_theme_has_icon              (GtkIconTheme                *icon_theme,
 						    const gchar                 *icon_name);
+gint         *gtk_icon_theme_get_icon_sizes        (GtkIconTheme                *icon_theme,
+						    const gchar                 *icon_name);
 GtkIconInfo * gtk_icon_theme_lookup_icon           (GtkIconTheme                *icon_theme,
 						    const gchar                 *icon_name,
 						    gint                         size,
