@@ -43,17 +43,25 @@ struct _GtkPathBar
   GtkFileSystem *file_system;
   GtkFilePath *root_path;
   GtkFilePath *home_path;
-  GdkPixbuf *home_icon;
+  GtkFilePath *desktop_path;
+
   GdkPixbuf *root_icon;
+  GdkPixbuf *home_icon;
+  GdkPixbuf *desktop_icon;
 
   GList *button_list;
   GList *first_scrolled_button;
   GtkWidget *up_slider_button;
   GtkWidget *down_slider_button;
+  guint settings_signal_id;
+  gint icon_size;
   gint16 slider_width;
   gint16 spacing;
   gint16 button_offset;
+  guint timer;
   guint slider_visible : 1;
+  guint need_timer : 1;
+  guint ignore_click : 1;
 };
 
 struct _GtkPathBarClass
@@ -61,7 +69,8 @@ struct _GtkPathBarClass
   GtkContainerClass parent_class;
 
   void (* path_clicked) (GtkPathBar  *path_bar,
-			 GtkFilePath *file_path);
+			 GtkFilePath *file_path,
+			 gboolean     child_is_hidden);
 };
 
 GType    gtk_path_bar_get_type (void) G_GNUC_CONST;
@@ -70,6 +79,8 @@ void     _gtk_path_bar_set_file_system (GtkPathBar         *path_bar,
 gboolean _gtk_path_bar_set_path        (GtkPathBar         *path_bar,
 					const GtkFilePath  *file_path,
 					GError            **error);
+void     _gtk_path_bar_up              (GtkPathBar *path_bar);
+void     _gtk_path_bar_down            (GtkPathBar *path_bar);
 
 G_END_DECLS
 
