@@ -56,8 +56,10 @@
 #include <string.h>
 #include "gdk.h"
 
+#include "gtkalias.h"
 #include "gtkmain.h"
 #include "gtkselection.h"
+#include "gdk-pixbuf/gdk-pixbuf.h"
 
 #ifdef GDK_WINDOWING_X11
 #include "x11/gdkx.h"
@@ -244,6 +246,7 @@ static GdkAtom ctext_atom;
 static GdkAtom text_plain_atom;
 static GdkAtom text_plain_utf8_atom;
 static GdkAtom text_plain_locale_atom;
+static GdkAtom text_uri_list_atom;
 
 static void 
 init_atoms (void)
@@ -262,21 +265,24 @@ init_atoms (void)
       tmp = g_strdup_printf ("text/plain;charset=%s", charset);
       text_plain_locale_atom = gdk_atom_intern (tmp, FALSE);
       g_free (tmp);
+
+      text_uri_list_atom = gdk_atom_intern ("text/uri-list", FALSE);
     }
 }
 
 /**
  * gtk_target_list_add_text_targets:
  * @list: a #GtkTargetList
+ * @info: an ID that will be passed back to the application
  * 
  * Adds the text targets supported by #GtkSelection to
- * the target list. The targets are added with both flags
- * and info being zero.
+ * the target list. All targets are added with the same @info.
  * 
  * Since: 2.6
  **/
 void 
-gtk_target_list_add_text_targets (GtkTargetList *list)
+gtk_target_list_add_text_targets (GtkTargetList *list,
+				  guint          info)
 {
   g_return_if_fail (list != NULL);
   
@@ -284,13 +290,78 @@ gtk_target_list_add_text_targets (GtkTargetList *list)
 
   /* Keep in sync with gtk_selection_data_targets_include_text()
    */
-  gtk_target_list_add (list, utf8_atom, 0, 0);  
-  gtk_target_list_add (list, ctext_atom, 0, 0);  
-  gtk_target_list_add (list, text_atom, 0, 0);  
-  gtk_target_list_add (list, GDK_TARGET_STRING, 0, 0);  
-  gtk_target_list_add (list, text_plain_utf8_atom, 0, 0);  
-  gtk_target_list_add (list, text_plain_locale_atom, 0, 0);  
-  gtk_target_list_add (list, text_plain_atom, 0, 0);  
+  gtk_target_list_add (list, utf8_atom, 0, info);  
+  gtk_target_list_add (list, ctext_atom, 0, info);  
+  gtk_target_list_add (list, text_atom, 0, info);  
+  gtk_target_list_add (list, GDK_TARGET_STRING, 0, info);  
+  gtk_target_list_add (list, text_plain_utf8_atom, 0, info);  
+  gtk_target_list_add (list, text_plain_locale_atom, 0, info);  
+  gtk_target_list_add (list, text_plain_atom, 0, info);  
+}
+
+/**
+ * gtk_target_list_add_image_targets:
+ * @list: a #GtkTargetList
+ * @info: an ID that will be passed back to the application
+ * @writable: whether to add only targets for which GTK+ knows
+ *   how to convert a pixbuf into the format
+ * 
+ * Adds the image targets supported by #GtkSelection to
+ * the target list. All targets are added with the same @info.
+ * 
+ * Since: 2.6
+ **/
+void 
+gtk_target_list_add_image_targets (GtkTargetList *list,
+				   guint          info,
+				   gboolean       writable)
+{
+  GSList *formats, *f;
+  gchar **mimes, **m;
+  GdkAtom atom;
+
+  g_return_if_fail (list != NULL);
+
+  formats = gdk_pixbuf_get_formats ();
+
+  for (f = formats; f; f = f->next)
+    {
+      GdkPixbufFormat *fmt = f->data;
+
+      if (writable && !gdk_pixbuf_format_is_writable (fmt))
+	continue;
+      
+      mimes = gdk_pixbuf_format_get_mime_types (fmt);
+      for (m = mimes; *m; m++)
+	{
+	  atom = gdk_atom_intern (*m, FALSE);
+	  gtk_target_list_add (list, atom, 0, info);  
+	}
+      g_strfreev (mimes);
+    }
+
+  g_slist_free (formats);
+}
+
+/**
+ * gtk_target_list_add_uri_targets:
+ * @list: a #GtkTargetList
+ * @info: an ID that will be passed back to the application
+ * 
+ * Adds the URI targets supported by #GtkSelection to
+ * the target list. All targets are added with the same @info.
+ * 
+ * Since: 2.6
+ **/
+void 
+gtk_target_list_add_uri_targets (GtkTargetList *list,
+				 guint          info)
+{
+  g_return_if_fail (list != NULL);
+  
+  init_atoms ();
+
+  gtk_target_list_add (list, text_uri_list_atom, 0, info);  
 }
 
 void               
@@ -1013,7 +1084,7 @@ selection_get_text_plain (GtkSelectionData *selection_data)
 {
   const gchar *charset = NULL;
   gchar *str, *result;
-  gint len;
+  gsize len;
   GError *error = NULL;
 
   str = g_strdup (selection_data->data);
@@ -1031,20 +1102,25 @@ selection_get_text_plain (GtkSelectionData *selection_data)
 				     charset, "UTF-8", 
 				     NULL, NULL, &len, &error);
       g_free (tmp);
-    }
 
-  if (!str)
+      if (!str)
+	{
+	  g_warning ("Error converting from %s to UTF-8: %s",
+		      charset, error->message);
+	  g_error_free (error);
+
+	  return NULL;
+	}
+    }
+  else if (!g_utf8_validate (str, -1, NULL))
     {
-      g_warning ("Error converting from %s to UTF-8: %s",
-		 charset, error->message);
-      g_error_free (error);
-      
+      g_warning ("Error converting from text/plain;charset=utf-8 to UTF-8");
+      g_free (str);
+
       return NULL;
     }
 
-
   result = normalize_to_lf (str, len);
-
   g_free (str);
 
   return result;
@@ -1151,6 +1227,201 @@ gtk_selection_data_get_text (GtkSelectionData *selection_data)
   return result;
 }
 
+/**
+ * gtk_selection_data_set_pixbuf:
+ * @selection_data: a #GtkSelectionData
+ * @pixbuf: a #GdkPixbuf
+ * 
+ * Sets the contents of the selection from a #GdkPixbuf
+ * The pixbuf is converted to the form determined by
+ * @selection_data->target.
+ * 
+ * Return value: %TRUE if the selection was successfully set,
+ *   otherwise %FALSE.
+ *
+ * Since: 2.6
+ **/
+gboolean
+gtk_selection_data_set_pixbuf (GtkSelectionData *selection_data,
+			       GdkPixbuf        *pixbuf)
+{
+  GSList *formats, *f;
+  gchar **mimes, **m;
+  GdkAtom atom;
+  gboolean result;
+  gchar *str, *type;
+  gsize len;
+
+  formats = gdk_pixbuf_get_formats ();
+
+  for (f = formats; f; f = f->next)
+    {
+      GdkPixbufFormat *fmt = f->data;
+
+      mimes = gdk_pixbuf_format_get_mime_types (fmt);
+      for (m = mimes; *m; m++)
+	{
+	  atom = gdk_atom_intern (*m, FALSE);
+	  if (selection_data->target == atom)
+	    {
+	      str = NULL;
+	      type = gdk_pixbuf_format_get_name (fmt),
+	      result = gdk_pixbuf_save_to_buffer (pixbuf,
+						  &str,
+						  &len,
+						  type,
+						  NULL);
+	      if (result) 
+		gtk_selection_data_set (selection_data,
+					atom, 8, (guchar *)str, len);
+	      g_free (type);
+	      g_free (str);
+	      g_strfreev (mimes);
+	      g_slist_free (formats);
+	      
+	      return result;
+	    }
+	}
+
+      g_strfreev (mimes);
+    }
+
+  g_slist_free (formats);
+ 
+  return FALSE;
+}
+
+/**
+ * gtk_selection_data_get_pixbuf:
+ * @selection_data: a #GtkSelectionData
+ * 
+ * Gets the contents of the selection data as a #GdkPixbuf.
+ * 
+ * Return value: if the selection data contained a recognized
+ *   image type and it could be converted to a #GdkPixbuf, a 
+ *   newly allocated pixbuf is returned, otherwise %NULL.
+ *   If the result is non-%NULL it must be freed with g_object_unref().
+ *
+ * Since: 2.6
+ **/
+GdkPixbuf *
+gtk_selection_data_get_pixbuf (GtkSelectionData *selection_data)
+{
+  GdkPixbufLoader *loader;
+  GdkPixbuf *result = NULL;
+
+  loader = gdk_pixbuf_loader_new ();
+  
+  if (gdk_pixbuf_loader_write (loader, 
+			       selection_data->data,
+			       selection_data->length,
+			       NULL))
+    result = gdk_pixbuf_loader_get_pixbuf (loader);
+  
+  if (result)
+    g_object_ref (result);
+
+  gdk_pixbuf_loader_close (loader, NULL);
+  g_object_unref (loader);
+  
+  return result;
+}
+
+/**
+ * gtk_selection_data_set_uris:
+ * @selection_data: a #GtkSelectionData
+ * @uris: a %NULL-terminated array of strings hilding URIs
+ * 
+ * Sets the contents of the selection from a list of URIs.
+ * The string is converted to the form determined by
+ * @selection_data->target.
+ * 
+ * Return value: %TRUE if the selection was successfully set,
+ *   otherwise %FALSE.
+ *
+ * Since: 2.6
+ **/
+gboolean
+gtk_selection_data_set_uris (GtkSelectionData  *selection_data,
+			     gchar            **uris)
+{
+  init_atoms ();
+
+  if (selection_data->target == text_uri_list_atom)
+    {
+      GString *list;
+      gint i;
+      gchar *result;
+      gsize length;
+      
+      list = g_string_new (NULL);
+      for (i = 0; uris[i]; i++)
+	{
+	  g_string_append (list, uris[i]);
+	  g_string_append (list, "\r\n");
+	}
+
+      result = g_convert (list->str, list->len,
+			  "ASCII", "UTF-8", 
+			  NULL, &length, NULL);
+      g_string_free (list, TRUE);
+      
+      if (result)
+	{
+	  gtk_selection_data_set (selection_data,
+				  text_uri_list_atom,
+				  8, (guchar *)result, length);
+	  
+	  return TRUE;
+	}
+    }
+
+  return FALSE;
+}
+
+/**
+ * gtk_selection_data_get_uris:
+ * @selection_data: a #GtkSelectionData
+ * 
+ * Gets the contents of the selection data as array of URIs.
+ * 
+ * Return value: if the selection data contains a list of
+ *   URIs, a newly allocated %NULL-terminated string array
+ *   containing the URIs, otherwise %NULL. If the result is 
+ *   non-%NULL it must be freed with g_strfreev().
+ *
+ * Since: 2.6
+ **/
+gchar **
+gtk_selection_data_get_uris (GtkSelectionData *selection_data)
+{
+  gchar **result = NULL;
+
+  init_atoms ();
+  
+  if (selection_data->length >= 0 &&
+      selection_data->type == text_uri_list_atom)
+    {
+      gchar **list;
+      gint i;
+      gint count = gdk_text_property_to_utf8_list_for_display (selection_data->display,
+      							       utf8_atom,
+						   	       selection_data->format, 
+						               selection_data->data,
+						               selection_data->length,
+						               &list);
+      if (count > 0)
+	result = g_uri_list_extract_uris (list[0]);
+      
+      for (i = 1; i < count; i++)
+	g_free (list[i]);
+      g_free (list);
+    }
+
+  return result;
+}
+
+
 /**
  * gtk_selection_data_get_targets:
  * @selection_data: a #GtkSelectionData object
@@ -1225,7 +1496,10 @@ gtk_selection_data_targets_include_text (GtkSelectionData *selection_data)
 	      targets[i] == text_plain_atom ||
 	      targets[i] == text_plain_utf8_atom ||
 	      targets[i] == text_plain_locale_atom)
-	    result = TRUE;
+	    {
+	      result = TRUE;
+	      break;
+	    }
 	}
 
       g_free (targets);
@@ -1233,6 +1507,54 @@ gtk_selection_data_targets_include_text (GtkSelectionData *selection_data)
 
   return result;
 }
+
+/**
+ * gtk_selection_data_targets_include_image:
+ * @selection_data: a #GtkSelectionData object
+ * @writable: whether to accept only targets for which GTK+ knows
+ *   how to convert a pixbuf into the format
+ * 
+ * Given a #GtkSelectionData object holding a list of targets,
+ * determines if any of the targets in @targets can be used to
+ * provide a #GdkPixbuf.
+ * 
+ * Return value: %TRUE if @selection_data holds a list of targets,
+ *   and a suitable target for images is included, otherwise %FALSE.
+ *
+ * Since: 2.6
+ **/
+gboolean 
+gtk_selection_data_targets_include_image (GtkSelectionData *selection_data,
+					  gboolean          writable)
+{
+  GdkAtom *targets;
+  gint n_targets;
+  gint i;
+  gboolean result = FALSE;
+  GtkTargetList *list;
+  GList *l;
+
+  init_atoms ();
+
+  if (gtk_selection_data_get_targets (selection_data, &targets, &n_targets))
+    {
+      list = gtk_target_list_new (NULL, 0);
+      gtk_target_list_add_image_targets (list, 0, writable);
+      for (i=0; i < n_targets && !result; i++)
+	{
+	  for (l = list->list; l && !result; l = l->next)
+	    {
+	      GtkTargetPair *pair = (GtkTargetPair *)l->data;
+	      if (pair->target == targets[i])
+		result = TRUE;
+	    }
+	}
+      gtk_target_list_unref (list);
+      g_free (targets);
+    }
+
+  return result;
+}
 	  
 /*************************************************************
  * gtk_selection_init:
